////////////////////////////////////////////////////////////////////////////////
// TTitleBar component
//
// Controls that allows to customise the look of the title-bar
//
// by Alan GARNY
//    gry@physiol.ox.ac.uk
//    http://www.physiol.ox.ac.uk/~gry
//
//  Copyright 1996-97
////////////////////////////////////////////////////////////////////////////////

#ifndef EnhancedFormH
#define EnhancedFormH

////////////////////////////////////////////////////////////////////////////////

class TTitleBar;

////////////////////////////////////////////////////////////////////////////////
// TSpecialCaption

class TSpecialCaption: public TPersistent
{
   private:
      // Private representation of published properties

      String FCaption;
      bool FEnabled;
      TFontStyles FFontStyle;

      // Methods to modify the different published properties

      void __fastcall SetCaption(const String aValue);
      void __fastcall SetEnabled(const bool aValue);
      void __fastcall SetFontStyle(const TFontStyles aValue);

      // Properties used for internal purpose

      TTitleBar *Owner;

   public:
      // Constructor & Destructor

      virtual __fastcall TSpecialCaption(TTitleBar *aOwner);

      inline void __fastcall Paint();

   __published:
      // Published properties

      __property String Caption = {read=FCaption, write=SetCaption, nodefault};
      __property bool Enabled = {read=FEnabled, write=SetEnabled, default=true};
      __property TFontStyles FontStyle = {read=FFontStyle, write=SetFontStyle, nodefault};
};

////////////////////////////////////////////////////////////////////////////////
// TTitleBar

//---GRY---enum TGradientType {gtNone, gtLeftToRight, gtRightToLeft, gtCenterToBorder, gtBorderToCenter};
enum TGradientType {gtNone, gtLeftToRight, gtCenterToBorder};

class TTitleBar: public TComponent
{
   private:
      // Private representation of published properties

      TAlignment FAlignment;
      TSpecialCaption *FApplicationInfo;
      TColor FColor;
      TSpecialCaption *FCompanyInfo;
      bool FEnabled;
      int FNbOfColors;
      TGradientType FType;

      // Methods to modify the different published properties

      void __fastcall SetAlignment(const TAlignment aValue);
      void __fastcall SetColor(const TColor aValue);
      void __fastcall SetEnabled(const bool aValue);
      void __fastcall SetNbOfColors(const int aValue);
      void __fastcall SetType(const TGradientType aValue);

      // Properties used for internal purpose

      TForm *Owner;
      HWND OwnerHandle;   // = Owner->Handle. Useful to have it, since we lose
                          // its original value when destroying the form, which
                          // results in some strange behaviours... :-(

      bool FormActive;
      bool ActiveDefined;

      String CrtCaption;   // To keep track of the current caption (useful when
                           // the user changes the form's caption)

      int (__stdcall *OldHookFormProc) ();
      int (__stdcall *NewHookFormProc) ();

      TFont *TitleBarFont;   // Windows' font used for the title-bar

      // Methods used for internal purpose

      void __fastcall GetTitleBarFont(TFont *aFont);

      void __fastcall HookForm();
      void __fastcall UnhookForm();

      void __fastcall HookFormProc(TMessage &aMsg);

      void __fastcall NCPaintHandler(TMessage &aMsg);
      void __fastcall NCActivateHandler(TMessage &aMsg);

      bool __fastcall FormIsActive();

      TRect __fastcall DrawTitleBar(const bool aActive);

      TRect __fastcall GetTitleBarCaptionRegion();
      TRect __fastcall GetTitleBarRegion();
      void __fastcall ExcludeButtonsRegion(TRect &aRegion);      

      void __fastcall GradientFill(HDC &aDC,
                                   const TColor &aBeginColor,
                                   const TColor &aEndColor,
                                   const TRect &aRegion);
      void __fastcall SolidFill(HDC &aDC, const TColor &aColor,
                                const TRect &aRegion);

      void __fastcall DrawSystemMenuIcon(HDC &aDC, TRect &aRegion);
      void __fastcall DrawTitleBarButtons(HDC &aDC, TRect &aRegion);

      TBorderIcons __fastcall GetVisibleButtons();

      long __fastcall CaptionWidth(HDC &aDC, TRect &aRegion,
                                   const String &aCaption,
                                   const TFontStyles &aFontStyle);
      void __fastcall DrawTitleBarCaption(HDC &aDC, TRect &aRegion,
                                          const String &aCaption,
                                          const TFontStyles &aFontStyle,
                                          const bool &aActive);

      // Overridden methods

      virtual void __fastcall Loaded();

   public:
      // Constructor & Destructor

      virtual __fastcall TTitleBar(TComponent *aOwner);
      virtual __fastcall ~TTitleBar();

      inline void __fastcall Paint();

      inline TForm* __fastcall GetOwner() { return Owner; }

   __published:
      // Published properties

      __property TAlignment Alignment = {read=FAlignment, write=SetAlignment, default=taLeftJustify};
      __property TSpecialCaption *ApplicationInfo = {read=FApplicationInfo, write=FApplicationInfo, nodefault};
      __property TColor Color = {read=FColor, write=SetColor, default=clBlack};
      __property TSpecialCaption *CompanyInfo = {read=FCompanyInfo, write=FCompanyInfo, nodefault};
      __property bool Enabled = {read=FEnabled, write=SetEnabled, default=true};
      __property int NbOfColors = {read=FNbOfColors, write=SetNbOfColors, default=64};
      __property TGradientType Type = {read=FType, write=SetType, default=gtLeftToRight};
};

////////////////////////////////////////////////////////////////////////////////

#endif

// End of file
////////////////////////////////////////////////////////////////////////////////
